@@ -19,8 +19,7 @@
 #' transformation when FALSE.
 #' @param OD Logical variable. When TRUE, and the output contains a
 #' column with heights, then a new column is added to the result indicating the
-#' ordnance datum (OD) used on each point. It is ignored when \code{OSTN=FALSE}
-#' or data doesn't contain 3D points.
+#' ordnance datum (OD) used on each point. It is ignored when \code{OSTN=FALSE}.
 #' @details
 #' The UK Ordnance Survey defined 'OSGB36' as the datum for the UK, based on the
 #' 'Airy 1830' ellipsoid. However, in 2014, they deprecated OSGB36 in favour of
@@ -80,6 +79,21 @@ sgo_lonlat_bng.sgo_points <- function(x, to=27700, OSTN=TRUE, OD=FALSE) {
   if (out.dimension == "XY/Z")
     out.dimension <- "XY"
 
+  # When converting from 2D to 3D, fill input z with 0's and a correct epsg
+  if (out.dimension == "XYZ" && !x.3d) {
+    # 4277 can't be converted to 3D
+    if (x$epsg == 4277) {
+      stop("Can't convert from EPSG:4277 (2D CS) to a 3D Coordinate System")
+    } else {
+      x$epsg <- .epsgs[.epsgs$datum==x$datum & .epsgs$type=="GCS" &
+                         .epsgs$dimension=="XYZ" & .epsgs$format=="ll", "epsg"]
+      x$dimension <- "XYZ"
+      x$z <- rep(0, length(x$x))
+      x.3d <- TRUE
+      #warning("Converted from 2D to 3D. Hence input heights default to 0")
+    }
+  }
+
   if (x.3d) {
     x.coords <- .sgo_points.3d.coords
     core.cols <- .sgo_points.3d.core
@@ -122,7 +136,7 @@ sgo_lonlat_bng.sgo_points <- function(x, to=27700, OSTN=TRUE, OD=FALSE) {
     # If datum is WGS84/ETRS89, we need to adjust with OSTN15
     if (x$epsg %in% c(4258, 4937)) {
 
-      shifts <- .find.OSTN.shifts.at(e, n, x.3d)
+      shifts <- .find.OSTN.shifts.at(e, n, x.3d, OD)
       # Round to mm precision
       e <- round(e + shifts$dx, 3)
       n <- round(n + shifts$dy, 3)
@@ -161,17 +175,12 @@ sgo_lonlat_bng.sgo_points <- function(x, to=27700, OSTN=TRUE, OD=FALSE) {
 
   # Return values with correct EPSG depending on input and output.
   if (out.dimension == "XYZ") {
-    if (OSTN && x.3d) {
-      if (OD) {
+    if (OSTN && OD) {
         en <- list(x=e, y=n, z=round(x$z - shifts$dz, 3),
                    height.datum=datum.flags[match(shifts$gf,
                                              datum.flags$geoid.datum.flag), 4])
-      } else {
-        en <- list(x=e, y=n, z=round(x$z - shifts$dz, 3))
-      }
     } else {
-      en <- list(x=e, y=n, z=rep(0, length(e)))
-      #warning("Converted from 2D to 3D. Hence heights default to 0")
+        en <- list(x=e, y=n, z=round(x$z - shifts$dz, 3))
     }
   } else {
     en <- list(x=e, y=n)
@@ -205,8 +214,7 @@ sgo_lonlat_bng.sgo_points <- function(x, to=27700, OSTN=TRUE, OD=FALSE) {
 #' transformation when FALSE.
 #' @param OD Logical variable. When TRUE, and the output contains a
 #' column with heights, then a new column is added to the result indicating the
-#' ordnance datum (OD) used on each point. It is ignored when \code{OSTN=FALSE}
-#' or data doesn't contain 3D points.
+#' ordnance datum (OD) used on each point. It is ignored when \code{OSTN=FALSE}.
 #' @details
 #' The UK Ordnance Survey defined 'OSGB36' as the datum for the UK, based on the
 #' 'Airy 1830' ellipsoid. However, in 2014, they deprecated OSGB36 in favour of
@@ -266,6 +274,15 @@ sgo_bng_lonlat.sgo_points <- function(x, to=4258, OSTN=TRUE, OD=FALSE) {
   if (out.dimension == "XY/Z")
     out.dimension <- "XY"
 
+  # When converting from 2D to 3D, fill input z with 0's and a correct epsg
+  if (out.dimension == "XYZ" && !has.z) {
+    x$z <- rep(0, length(x$x))
+    x$epsg <- 7405
+    x$dimension <- "XYZ"
+    has.z <- TRUE
+    #warning("Converted from 2D to 3D. Hence input heights default to 0")
+  }
+
   if (has.z) {
     core.cols <- .sgo_points.3d.core
   } else {
@@ -284,14 +301,14 @@ sgo_bng_lonlat.sgo_points <- function(x, to=4258, OSTN=TRUE, OD=FALSE) {
 
     if (to %in% c(4258, 4937, 4326, 4979)) {
 
-      shifts <- .find.OSTN.shifts.at(x$x, x$y, has.z)
+      shifts <- .find.OSTN.shifts.at(x$x, x$y, has.z, OD)
       e <- x$x - shifts$dx
       n <- x$y - shifts$dy
       last.shifts <- shifts
 
       for (i in c(1:20)) {
 
-        shifts <- .find.OSTN.shifts.at(e, n, has.z)
+        shifts <- .find.OSTN.shifts.at(e, n, has.z, OD)
         if (all(shifts$out) == TRUE) {
           # all coordinates have been shifted off the edge
           break
@@ -344,21 +361,14 @@ sgo_bng_lonlat.sgo_points <- function(x, to=4258, OSTN=TRUE, OD=FALSE) {
 
 
   if (out.dimension == "XYZ") {
-    if (OSTN && has.z) {
-      if (OD) {
-
+    if (OSTN && OD) {
         unprojected <- list(x=unprojected[, 1], y=unprojected[, 2],
                             z=round(x$z + shifts$dz, 4),
                             height.datum=datum.flags[match(shifts$gf,
                                             datum.flags$geoid.datum.flag), 4])
-      } else {
+    } else {
         unprojected <- list(x=unprojected[, 1], y=unprojected[, 2],
                             z=round(x$z + shifts$dz, 4))
-      }
-    } else {
-      unprojected <- list(x=unprojected[, 1], y=unprojected[, 2],
-                          z=rep(0, length(unprojected[, 1])))
-      #warning("Converted from 2D to 3D. Hence heights default to 0")
     }
   } else {
     unprojected <- list(x=unprojected[, 1], y=unprojected[, 2])
@@ -534,8 +544,9 @@ sgo_bng_lonlat.sgo_points <- function(x, to=4258, OSTN=TRUE, OD=FALSE) {
 #' @noRd
 #' @param e A numeric vector with Easting coordinates
 #' @param n A numeric vector with Northing coordinates
-#' @param z A numeric vector with Height coordinates
-.find.OSTN.shifts.at <- function(e, n, z=FALSE) {
+#' @param z A logical value indicating whether we need to compute heights
+#' @param flag A logical value indicating whether we need the OD as output
+.find.OSTN.shifts.at <- function(e, n, z=FALSE, flag=FALSE) {
 
   # Initialise list of shifts
   len.e <- length(e)
@@ -550,8 +561,10 @@ sgo_bng_lonlat.sgo_points <- function(x, to=4258, OSTN=TRUE, OD=FALSE) {
   }
 
   # OSTN15 covers grid point (0, 0) to (700000, 1250000)
-  out.of.bounds <- (e < 0 | e > 700000) |
-                   (n < 0 | n > 1250000) #| (is.na(e) | is.na(n))
+  # NA's are checked in case there are stored coordinates as BNG that are
+  # actually outside of the OSTN15 rectangle.
+  out.of.bounds <- (e < 0 | e >= 700000) |
+                   (n < 0 | n >= 1250000) | (is.na(e) | is.na(n))
   shifts$out <- out.of.bounds
 
 
@@ -593,17 +606,22 @@ sgo_bng_lonlat.sgo_points <- function(x, to=4258, OSTN=TRUE, OD=FALSE) {
              + one.t * u * ul[, "g"]
              + t * u * ur[, "g"])
 
-      llf <- ll[, "f"]
-      lrf <- lr[, "f"]
-      ulf <- ul[, "f"]
-      urf <- ur[, "f"]
-      gf <- ifelse(llf == lrf & lrf == ulf & ulf == urf, llf #all equal
-                 , ifelse(t <= 0.5 & u <= 0.5, llf #point in SW (or dead centre)
-                 , ifelse(t > 0.5 & u <= 0.5, lrf #point in SE quadrant
-                 , ifelse(t > 0.5 & u > 0.5, urf  #point in NE quadrant
-                 , ulf))))
       shifts$dz[!out.of.bounds] <- dz
-      shifts$gf[!out.of.bounds] <- gf
+
+      if (flag) {
+        llf <- ll[, "f"]
+        lrf <- lr[, "f"]
+        ulf <- ul[, "f"]
+        urf <- ur[, "f"]
+
+        gf <- .if.else(llf == lrf & lrf == ulf & ulf == urf, llf #all equal
+                   , .if.else(t <= 0.5 & u <= 0.5, llf #point in SW (or dead centre)
+                   , .if.else(t > 0.5 & u <= 0.5, lrf #point in SE quadrant
+                   , .if.else(t > 0.5 & u > 0.5, urf  #point in NE quadrant
+                   , ulf))))
+
+        shifts$gf[!out.of.bounds] <- gf
+      }
     }
 
   }
@@ -611,3 +629,19 @@ sgo_bng_lonlat.sgo_points <- function(x, to=4258, OSTN=TRUE, OD=FALSE) {
   return (shifts)
 
 }
+
+
+# Helper function. Slightly faster ifelse variant
+#' @noRd
+#' @param test An object which can be coerced to logical mode
+#' @param yes Return values for true elements of test
+#' @param no Return values for true elements of test
+.if.else <- function(test, yes, no){
+
+  out <- rep(NA, length(test))
+  out[test] <- yes[test]
+  out[!test] <- no[!test]
+
+  out
+
+}
